-- <ScriptOptions statementTerminator=";" />
		
CREATE TABLE METADATASOURCE_T (
        METADATASRC_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		SOURCE VARCHAR(255),
        IMPORT_TIME TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
        CONTENT_LENGTH INTEGER,
		CONTENT BLOB(1G),
		SOURCEFILE VARCHAR(2048)
);

-- Hold information about the metadata source files
-- Each row is one source file that was loaded.
CREATE VIEW METADATASOURCE AS SELECT
		METADATASRC_KEY,                      -- Key for this source entry
		SOURCE,                               -- Type of file:  'Analysis', 'PureQuery', etc
		IMPORT_TIME,                          -- Time file was imported
		CONTENT_LENGTH,                       -- The uncompressed length of the content
		CONTENT,                              -- Content of the file
		SOURCEFILE                            -- File path & name
		FROM METADATASOURCE_T;   
	
			
CREATE TABLE CAPTUREDATA_T (
        CAPTUREDATA_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
		GROUP_NAME VARCHAR(255) NOT NULL,
		GROUP_VERSION VARCHAR(255), 
        UPDATE_TIME TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
		CONTENT BLOB(1G),
		CONTENT_LENGTH INTEGER,
		STATE   VARCHAR(255),
		STATE_TRANSITION_TIME TIMESTAMP
);	

-- Holds the content of incrementally captured data files
CREATE VIEW CAPTUREDATA AS SELECT
		CAPTUREDATA_KEY,                      -- Key 
		GROUP_NAME,                           -- Maps to NAME in METADATAGROUPS_T table
		GROUP_VERSION,                        -- Maps to VERSION in METADATAGROUPS_T table
		UPDATE_TIME,                          -- Time file was last updated
		CONTENT,                              -- Content of the file
		CONTENT_LENGTH,                       -- Uncompressed length of the content
		STATE,                                -- The current state of the data
		STATE_TRANSITION_TIME                 -- The time the current state was set
		FROM CAPTUREDATA_T;    
		
CREATE TABLE METADATAGROUP_T (
        METADATAGROUP_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
        NAME        VARCHAR(255) NOT NULL,       
        VERSION     VARCHAR(255),
        ACTIVE      CHAR(1),       
        TYPE_       CHAR(1),
        CONTACT_KEY INTEGER,
-- The following was added in 2.2.7; don't include in VIEW METADATAGROUP.
        ASYNC_REFRESH_TRACK INTEGER
);

-- The definition of all the groups. ID + VERSION must be unique 
CREATE VIEW METADATAGROUP AS SELECT 
        METADATAGROUP_KEY,                    -- Key for this entry      
        NAME,                                 -- The external ID
        VERSION,                              -- The version of this ID
        ACTIVE,                               -- 'Y' or 'N'
        TYPE_,                                -- 'S' for SQLGroups, 'R' for RuntimeGroups
        CONTACT_KEY                           -- Key into CONTACT table for any contact information.
        FROM METADATAGROUP_T;
        
CREATE TABLE METADATAGROUPMEMBERS_T (
        METADATAGROUP_KEY INTEGER NOT NULL,
        MEMBER_TYPE CHAR(1) NOT NULL,  
        METADATASRC_KEY INTEGER,       
        CHILDGROUP_KEY INTEGER,
        CHILDGROUP_NAME VARCHAR(255),
        CHILDGROUP_TYPE CHAR(1)
);

--  Holds the list of group members
--  Each row is one member of the group.  Although the row has nullable keys, one is always
--  non null and the others null depending on what type of member is in the group. 
--     If the member is a file, then METADATASRC_KEY is not null.
--     If the member is another specific group/version, then CHILDGROUP_KEY is not null.
--     If the member are all versions of a specific group, then CHILDGROUP_NAME and CHILDGROUP_TYPE are not null.
CREATE VIEW METADATAGROUPMEMBERS AS SELECT 
        METADATAGROUP_KEY,                    -- Key into METADATAGROUP_T table, holds what group id of this member.
        MEMBER_TYPE,                          -- 'C' for capture.pdqxml files, 'B' for bindProps, 'G' for genProps, 'P' for pdq.properties, 'S' for sql group  
        METADATASRC_KEY,                      -- Key into METADATASRC_T table, holds what file is part of the group.
        CHILDGROUP_KEY,                       -- Key into METADATAGROUP_T table, holds what child group is part of the group.
        CHILDGROUP_NAME,                      -- Name into METADATAGROUP_T table, holds the name of the child group.
        CHILDGROUP_TYPE                       -- Type into METADATAGROUP_T table.  Used with CHILDGROUP_NAME
        FROM METADATAGROUPMEMBERS_T;
        
CREATE TABLE CONTACT_T (
        CONTACT_KEY INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY,
	    CONTACT VARCHAR(255)
);

-- Information about contacts responsible for the metadata
CREATE VIEW CONTACT AS SELECT 
        CONTACT_KEY,                          -- Table key
        CONTACT                               -- Information about the contact
        FROM CONTACT_T;

	
CREATE TABLE SCHEMA_VERSION_T (
		COMPONENT VARCHAR(255),
		VERSION INTEGER,
		RELEASE INTEGER,
		MOD		INTEGER,
-- The following were added in 2.2.7; don't include in VIEW SCHEMA_VERSION.
		UUID_MOST_SIGNIFICANT_BITS BIGINT,
        UUID_LEAST_SIGNIFICANT_BITS BIGINT		
	);

-- Metadata schema information	
-- Each row holds information about one specific component.
CREATE VIEW SCHEMA_VERSION AS SELECT
	COMPONENT,                                  -- Name of the component.  Ex: com.ibm.pureQuery
	VERSION,                                    -- Version #.  Ex for v1.2.3, this is 1
	RELEASE,                                    -- Release #   Ex for v1.2.3, this is 2
	MOD                                         -- Modification #  Ex for v1.2.3, this is 3
    FROM SCHEMA_VERSION_T;
	

ALTER TABLE METADATASOURCE_T ADD CONSTRAINT METADATASRC_T_PK PRIMARY KEY(METADATASRC_KEY);

ALTER TABLE METADATAGROUP_T ADD CONSTRAINT MDGROUP_T_PK PRIMARY KEY(METADATAGROUP_KEY);

ALTER TABLE CONTACT_T ADD CONSTRAINT CONTACT_T_PK PRIMARY KEY(CONTACT_KEY);

ALTER TABLE CAPTUREDATA_T ADD CONSTRAINT CAPTUREDATA_T_PK PRIMARY KEY(CAPTUREDATA_KEY);

	
ALTER TABLE METADATAGROUP_T ADD CONSTRAINT MDG_CONTACT_T_FK FOREIGN KEY (CONTACT_KEY)
    REFERENCES CONTACT_T (CONTACT_KEY)
    ON DELETE CASCADE;
    
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDG_T_FK FOREIGN KEY (METADATAGROUP_KEY)
    REFERENCES METADATAGROUP_T (METADATAGROUP_KEY)
    ON DELETE CASCADE;
    
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDG_2_T_FK FOREIGN KEY (CHILDGROUP_KEY)
    REFERENCES METADATAGROUP_T (METADATAGROUP_KEY)
    ON DELETE CASCADE;
       	
ALTER TABLE METADATAGROUPMEMBERS_T ADD CONSTRAINT MDGM_MDS_T_FK FOREIGN KEY (METADATASRC_KEY)
    REFERENCES METADATASOURCE_T (METADATASRC_KEY)
    ON DELETE CASCADE;
    
CREATE UNIQUE INDEX METADATAGROUP_T_IDX ON METADATAGROUP_T (NAME, VERSION, TYPE_);
    
INSERT INTO SCHEMA_VERSION_T (COMPONENT, VERSION, RELEASE, MOD) values ('com.ibm.pureQuery.lite', 2, 2, 4);
	
{upgrades};
	
